/* 
  project "Evaluations-Platine für Heimautomatisierung"
  created 2013 by Thomas Gauweiler
  copyright: Apache License 2.0, http://www.apache.org/licenses/LICENSE-2.0.html
*/
#define SKETCH_NAME "BUTTONS_AND_LEDS"
#define PIN_BUTTONS A5
#define PIN_BUTTON4 3
#define PIN_RED 6
#define PIN_YELLOW 5
#define PIN_GREEN 4

#define BUTTON_1    184
#define BUTTON_2    252
#define BUTTON_3    328
#define BUTTON_12   118
#define BUTTON_13   132
#define BUTTON_23   165
#define BUTTON_123   94
#define NOBUTTON   1023

#define RED_ON LOW
#define RED_OFF HIGH
#define YELLOW_ON HIGH
#define YELLOW_OFF LOW
#define GREEN_ON HIGH
#define GREEN_OFF LOW

void switch_all_leds_on() {
  digitalWrite(PIN_RED, RED_ON);
  digitalWrite(PIN_YELLOW, YELLOW_ON);
  digitalWrite(PIN_GREEN, GREEN_ON);
}

void switch_all_leds_off() {
  digitalWrite(PIN_RED, RED_OFF);
  digitalWrite(PIN_YELLOW, YELLOW_OFF);
  digitalWrite(PIN_GREEN, GREEN_OFF);
}

void setup_serial() {
  Serial.begin(9600);
  Serial.print(SKETCH_NAME);
  Serial.println(" is ready");
}

void setup_leds() {
  pinMode(PIN_RED, OUTPUT);
  pinMode(PIN_YELLOW, OUTPUT);
  pinMode(PIN_GREEN, OUTPUT);
  switch_all_leds_on();
  delay(1000);
  switch_all_leds_off();
  delay(1000);
}

void setup_button() {
  pinMode(PIN_BUTTON4, INPUT);
}

void setup() {
  setup_serial();
  setup_leds();
  setup_button();
}


/******************************************/

bool is_nearly(const int requirement, const int value) {
  return (requirement - 10 < value && value < requirement + 10);
}

bool is_only_button1(const int value) {
  return (is_nearly(BUTTON_1, value));
}

bool is_only_button2(const int value) {
  return (is_nearly(BUTTON_2, value));
}

bool is_only_button3(const int value) {
  return (is_nearly(BUTTON_3, value));
}

bool is_button12(const int value) {
  return (is_nearly(BUTTON_12, value));
}

bool is_button13(const int value) {
  return (is_nearly(BUTTON_13, value));
}

bool is_button23(const int value) {
  return (is_nearly(BUTTON_23, value));
}

bool is_button123(const int value) {
  return (is_nearly(BUTTON_123, value));
}

bool is_button1(const int value) {
  return (is_only_button1(value) || is_button12(value) || is_button13(value) || is_button123(value));
}

bool is_button2(const int value) {
  return (is_only_button2(value) || is_button12(value) || is_button23(value) || is_button123(value));
}

bool is_button3(const int value) {
  return (is_only_button3(value) || is_button13(value) || is_button23(value) || is_button123(value));
}

void switch_red(const int value) {
  if (is_button1(value)) 
    digitalWrite(PIN_RED, RED_ON);
  else
    digitalWrite(PIN_RED, RED_OFF);
}

void switch_yellow(const int value) {
  if (is_button2(value)) 
    digitalWrite(PIN_YELLOW, YELLOW_ON);
  else
    digitalWrite(PIN_YELLOW, YELLOW_OFF);
}

void switch_green(const int value) {
  if (is_button3(value)) 
    digitalWrite(PIN_GREEN, GREEN_ON);
  else
    digitalWrite(PIN_GREEN, GREEN_OFF);
}

void digital_button() {
  bool buttonState;
  buttonState = digitalRead(PIN_BUTTON4);
  if (!buttonState) {
    switch_all_leds_on();
  }
}

void loop()
{
  int buttonsValue = 0;
  buttonsValue = analogRead(PIN_BUTTONS);
  Serial.print(" B1: ");   Serial.print(is_button1(buttonsValue));
  Serial.print(" B2: ");   Serial.print(is_button2(buttonsValue));
  Serial.print(" B3: ");   Serial.print(is_button3(buttonsValue));
  Serial.print(" - ");     Serial.println(buttonsValue);
  switch_red(buttonsValue);
  switch_yellow(buttonsValue);
  switch_green(buttonsValue);
  digital_button();
  delay (800);
}

